﻿using System;
using System.Runtime.Serialization;
using Flatland.GeneticProgramming.CoreTypes;

/**
* 
* Copyright (c) 2012 Anders Høst Kjærgaard | ahkj@itu.dk
* 
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
* associated documentation files (the "Software"), to deal in the Software without restriction, 
* including without limitation the rights to use, copy, modify, merge, publish, distribute, 
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial 
* portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
* NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
namespace Flatland.GeneticProgramming.Examples {

    /// <summary>
    /// Example of a GeneCommand implementation. GeneCommand is used for problems where we want the genetic code
    /// to make command calls to our gene. An example of this could be navigation instuctions to an Agent - void Move(), void Turn() etc.
    /// GeneCommand genetic code consists a sequence of statements where some will be conditionals (if_then_else).
    /// GeneCommand can only Primitive tag members of type bool or void.
    /// </summary>
    [Serializable]
    public class CommandExample : GenoType {

        private int _position;

        [Primitive]
        public bool GoodPosition() {
            Console.WriteLine("Good position: {0}", Math.Abs(_position) < 3);
            return Math.Abs(_position) < 3;
        }

        [Primitive]
        public void MoveLeft() {
            Console.WriteLine("Moving left");
            --_position;
        }

        [Primitive]
        public void MoveRight() {
            Console.WriteLine("Moving right");
            ++_position;
        }

        /// <summary>
        /// Goal is to end up in position 0;
        /// </summary>
        /// <returns></returns>
        public override double Fitness() {
            Execute();
            int fitness = GeneticProgramming.GP.DEFAULT_FITNESS_GOAL - _position;
            Console.WriteLine("Fitness of CommandExample: {0}", fitness);
            return 42;// fitness;
        }

        public override object Clone() {
            return new CommandExample();
        }


        /// <summary>
        /// From base class Behavior. The the GenoType is asked to act.
        /// This is often sone by simply calling Execute()
        /// </summary>
        protected override void Act(ref string msg) {
            Execute();
        }

           public CommandExample(SerializationInfo info, StreamingContext context) {}
        public CommandExample()
        {
        }
        
    }
}
