﻿using System;
using FarseerPhysics.Common;
using Flatland.Utilities;
using Microsoft.Xna.Framework;

/**
* 
* Copyright (c) 2012 Anders Høst Kjærgaard | ahkj@itu.dk
* 
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
* associated documentation files (the "Software"), to deal in the Software without restriction, 
* including without limitation the rights to use, copy, modify, merge, publish, distribute, 
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial 
* portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
* NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
* WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

namespace Flatland.Geomerty {

    /// <summary>
    /// Shaper can create geometrical shapes for the Flantlanders.
    /// </summary>
    class Shaper {
        /// <summary>
        /// Returns a list of vertices equally distanced on the unity circle.
        /// </summary>
        /// <param name="vertices">Number of vertices</param>
        /// <param name="radians"> </param>
        /// <returns></returns>
        public static Vertices FlatlanderShape(int vertices, double radians) {
            double vertDist = (2 * Math.PI) / vertices;
            double first = Math.Sin(radians);
            var verts = new Vertices(vertices);
            for (int i = 0; i < vertices; i++) {
                verts.Add(new Vector2((float)Math.Cos(first + i * vertDist), -(float)Math.Sin(first + i * vertDist)));
            }
            return verts;
        }
    }
}
