﻿using System;
using Flatland.GeneticProgramming.Compilation;
using Flatland.GeneticProgramming.CoreTypes;
using Flatland.GeneticProgramming.Initializations;
using Flatland.GeneticProgramming.SyntaxTree;
using Flatland.GeneticProgramming.TestProblems_Do_not_modify;
using Flatland.GeneticProgramming.TestProblems_Do_not_modify_;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test {
    /// <summary>
    /// Test class for operations on the Tree.
    /// </summary>
    [TestClass]
    public class InitializationTest : TestHelper {


        [TestMethod]
        public void ThreeDepthTrees(){
            Compiler comp = new Compiler("ThreeTreeTest_output");
            foreach (InitializationTechnique initializationTechnique in Enum.GetValues(typeof(InitializationTechnique))) {
                //GenoType 1
                CommandTestGene prop = new CommandTestGene();
                PrimitiveSet primitiveSet = new PrimitiveSet();
                primitiveSet.Parse(prop);
                Initialization init = new Initialization(initializationTechnique);
                for (int i = 0; i < 4; i++)
                {
                    Tree t1 = init.Init(3, primitiveSet);
                    prop.Tree = t1;
                    Console.WriteLine("Init method: {0}\nTree:  {1}", initializationTechnique, i);
                    t1.PrintTree();
                    Report r = new Report();
                    r.PrimitiveSet = primitiveSet;
                    r.GenoType = prop;
                    comp.Compile(r, "ThreeTreeTest" + initializationTechnique + "_" + i, true);
                }

            }
        } 


        /// <summary>
        /// Test Full initialization class class for building two trees of type GENE with depth 1 to 6. 
        /// For this the depth of the two trees should always be the same.
        /// All non root nodes must have a parent.
        /// No node refernce must exist in both trees at the same time.
        /// Print mehthod for visual inspection of the tree structure.
        /// </summary>
        [TestMethod]
        public void FullInitDepth() {
            //GeneCommand
            TestGeneCommand tc = new TestGeneCommand();
            PrimitiveSet primitiveSet = new PrimitiveSet();
            primitiveSet.Parse(tc);

            //Similar GeneCommand
            TestGeneCommand tc2 = new TestGeneCommand();
            PrimitiveSet DNA2 = new PrimitiveSet();
            DNA2.Parse(tc2);

            Console.WriteLine("Building trees for {0} and {1}", tc, tc2);
            for (int depth = 1; depth < 6; depth++) {
                Tree t1 = new FullInitialization().Init((uint)depth, primitiveSet);
                Assert.AreEqual(depth, t1.MaxDetpth(), "Depth desired" + depth + " Actual depth " + t1.MaxDetpth());

                Tree t2 = new FullInitialization().Init((uint)depth, DNA2);
                Assert.AreEqual(depth, t2.MaxDetpth());
                CheckAllHasParent(t1);
                CheckAllHasParent(t2);
                Assert.IsFalse(CheckHasSharedReference(t1, t2), "The same node was found in both trees");

                Console.WriteLine("Init trees");
                t1.PrintTree();
                Console.WriteLine("Tree 2: ");
                t2.PrintTree();
            }
        }

        /// <summary>
        /// Check initialized trees are valid
        /// Prints trees for visual inspection.
        /// </summary>
        [TestMethod]
        public void BuildTrees() {
            foreach (InitializationTechnique initializationTechnique in Enum.GetValues(typeof(InitializationTechnique)))
            {
                //GenoType 1
                Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test prop = new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test();
                PrimitiveSet primitiveSet = new PrimitiveSet();
                primitiveSet.Parse(prop);

                //GenoType 2 with different function names
                TestGene3 prop2 = new TestGene3();
                PrimitiveSet prim2 = new PrimitiveSet();
                prim2.Parse(prop2);

                for (int depth = 2; depth < 10; depth++)
                {

                    Tree t1 = new Initialization(initializationTechnique).Init((uint) depth, primitiveSet);
                    Tree t2 = new Initialization(initializationTechnique).Init((uint) depth, prim2);
                    CheckAllHasParent(t1);
                    CheckAllHasParent(t2);
                    Assert.IsFalse(CheckHasSharedReference(t1, t2), "The same node was found in both trees");

                    Console.WriteLine("Init trees");
                    t1.PrintTree();
                    Console.WriteLine("Tree 2: ");
                    t2.PrintTree();
                }
            }

        }
    }
}
