﻿using System;
using System.Collections.Generic;
using Flatland.GeneticProgramming.Compilation;
using Flatland.GeneticProgramming.CoreTypes;
using Flatland.GeneticProgramming.Initializations;
using Flatland.GeneticProgramming.Mutations;
using Flatland.GeneticProgramming.SyntaxTree;
using Flatland.GeneticProgramming.TestProblems_Do_not_modify_;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test {
    [TestClass]
    public class MutationTest : TestHelper {

        /// <summary>
        /// After a mutation node(s) in the tree should have been replaceed by members of the primitives sets in the PrimitiveSet - possibly leaving the 
        /// tree identacal.
        /// No nodes must exist in two trees.
        /// Children must have a parent that knows them.
        /// Primitives must be with correct nodes - primitives in terminals, and functions in nodes.
        /// Trees are never deeper than max depth
        /// </summary>
        [TestMethod]
        public void MutationsTestExpre()
        {
            foreach (InitializationTechnique initmethod in Enum.GetValues(typeof (InitializationTechnique)))
            {
                Initialization init = new Initialization(initmethod);
                Compiler comp = new Compiler("Mutation_Output");
                foreach (MutationTechnique mutationMethod in Enum.GetValues(typeof (MutationTechnique)))
                {
                    for (int treedepth = 1; treedepth < 5; treedepth++)
                    {
                        GenoType gene = new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test();
                        PrimitiveSet primitiveSet = new PrimitiveSet();
                        primitiveSet.Parse(gene);
                        Tree t1 = init.Init((uint) treedepth, primitiveSet);

                        GenoType gene2 = new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test();
                        PrimitiveSet dna2 = new PrimitiveSet();
                        dna2.Parse(gene2);
                        Tree t2 = init.Init((uint) treedepth, dna2);

                        Console.WriteLine("EXPRESSION GenoType {0} treedepth {1}", gene, treedepth);
                        Console.WriteLine("Mutation method: {0} Initialization method: {1}", mutationMethod, initmethod);
                        Console.WriteLine("Before Mutation");
                        Console.WriteLine("Tree 1:");
                        t1.PrintTree();
                        Console.WriteLine("Tree 2:");
                        t2.PrintTree();
                        Report r1 = new Report {GenoType = gene, PrimitiveSet = primitiveSet};
                        r1.GenoType.Tree = t1;
                        Report r2 = new Report(){GenoType = gene2, PrimitiveSet = dna2};
                        r2.GenoType.Tree = t2;
                        new Mutation(mutationMethod).Mutate(new List<Report>{r1}, 1, treedepth);
                        new Mutation(mutationMethod).Mutate(new List<Report> {r2}, 1, treedepth);
                        Console.WriteLine("AFTER Mutation:");
                        t1.PrintTree();
                        t2.PrintTree();

                        Console.WriteLine("");
                        Console.WriteLine("***************************");
                        Console.WriteLine("");

                        CheckAllHasParent(t1);
                        CheckAllHasParent(t2);
                        CheckCorrectPrimitivesInNodes(t1);
                        CheckCorrectPrimitivesInNodes(t2);
                        CheckPopulationForUniqueNodes(new List<Report>() {r1, r2});
                        Assert.IsTrue(t1.MaxDetpth() <= treedepth);
                        Assert.IsTrue(t2.MaxDetpth() <= treedepth);
                    }
                }
            }
        }


        /// <summary>
        /// After a mutation node(s) in the tree should have been replaceed by members of the primitives sets in the PrimitiveSet - possibly leaving the 
        /// tree identacal.
        /// No nodes must exist in two trees.
        /// Children must have a parent that knows them.
        /// Primitives must be with correct nodes - primitives in terminals, and functions in nodes.
        /// Trees are never deeper than max depth
        /// </summary>
        [TestMethod]
        public void MutationsTestCmd() {
            foreach (InitializationTechnique initializationTechnique in Enum.GetValues(typeof(InitializationTechnique))) {
                Initialization init = new Initialization(initializationTechnique);
                Compiler comp = new Compiler("Mutation_Output");
                foreach (MutationTechnique mutationTechnique in Enum.GetValues(typeof(MutationTechnique))) {
                    for (int treedepth = 1; treedepth < 5; treedepth++) {
                        GenoType gene = new CommandTestGene();
                        PrimitiveSet primitiveSet = new PrimitiveSet();
                        primitiveSet.Parse(gene);
                        Tree t1 = init.Init((uint)treedepth, primitiveSet);

                        GenoType gene2 = new CommandTestGene();
                        PrimitiveSet prim2 = new PrimitiveSet();
                        prim2.Parse(gene2);
                        Tree t2 = init.Init((uint)treedepth, prim2);

                        for (int i = 0; i < 10; i++)
                        {
                            Console.WriteLine("COMMAND GenoType {0} treedepth {1} Generation: {2}", gene, treedepth, i);
                            Console.WriteLine("Mutation method: {0} Initialization method: {1}", mutationTechnique, initializationTechnique);
                            Console.WriteLine("Before Mutation");
                            Console.WriteLine("Tree 1:");
                            t1.PrintTree();
                            Console.WriteLine("Tree 2:");
                            t2.PrintTree();
                            Report r1 = new Report { GenoType = gene, PrimitiveSet = primitiveSet };
                            r1.GenoType.Tree = t1;
                            Report r2 = new Report() { GenoType = gene2, PrimitiveSet = prim2 };
                            r2.GenoType.Tree = t2;
                            new Mutation(mutationTechnique).Mutate(new List<Report> { r1 }, 1, treedepth);
                            new Mutation(mutationTechnique).Mutate(new List<Report> { r2 }, 1, treedepth);
                            Console.WriteLine("AFTER Mutation:");
                            t1.PrintTree();
                            t2.PrintTree();

                            Console.WriteLine("");
                            Console.WriteLine("***************************");
                            Console.WriteLine("");

                            CheckAllHasParent(t1);
                            CheckAllHasParent(t2);
                            CheckCorrectPrimitivesInNodes(t1);
                            CheckCorrectPrimitivesInNodes(t2);
                            CheckPopulationForUniqueNodes(new List<Report>() { r1, r2 });
                            Assert.IsTrue(t1.MaxDetpth() <= treedepth);
                            Assert.IsTrue(t2.MaxDetpth() <= treedepth); 
                        }
                    }
                }
            }
        }
    }
}
