﻿using System;
using Flatland.GeneticProgramming.CoreTypes;
using Flatland.GeneticProgramming.TestGenes_Do_not_modify_;
using Flatland.GeneticProgramming.TestProblems_Do_not_modify;
using Flatland.GeneticProgramming.TestProblems_Do_not_modify_;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test {
    /// <summary>
    /// Test of the PrimitiveSet.
    /// </summary>
    [TestClass]
    public class PrimitivesTest {
        /// <summary>
        /// Execute the PrimitiveSet test. If these tests fail, one would want to check that the test classes 
        /// attributed the number of members as we are assuming here.
        /// </summary>
        [TestMethod]
        public void DnaParseTest()
        {
            DnaParseExpressionTestHelper(new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test(), 11, 5);
            DnaParseExpressionTestHelper(new TestGene4(), 1, 1);
            DnaParseExpressionTestHelper(new EmptyTestExpression(), 0, 0);

            DnaParseCommandTestHelper(new CommandTestGene(), 3, 2, 3);
            DnaParseCommandTestHelper(new EmptyCommandgene(), 0, 0, 0);
            DnaParseCommandTestHelper(new CommandTestGene2(), 1, 0, 1);

            DnaParseExpressionTestHelper(new FactoredDna(), 42, 17);
            DnaParseExpressionTestHelper(new FactoredDna2(), 43, 18);
        }

        /// <summary>
        /// A test prob like Test should yield 11 terminals and 5 functions. In Test two 
        /// addtional members (a function and a terminal) are attributed with PrimitiveSet but wiht the wrong closure type -
        /// these should not be in the function and terminal sets.
        /// 
        /// Features of the PrimitiveSet should not be null after a parse.
        /// 
        /// Closure type should be that of the generic parameter. Here the type is hardcoded in for the Test.
        /// 
        /// Boolean sets can exist, but must be empty. 
        /// 
        /// Test has PrimitiveSet attributed members that should be ignored becasu of their type
        /// 
        /// Typed sets should contain only unique objects and the amount of terminals and functions attributed with PrimitiveSet respectivly.
        /// <param name="testGene"></param>
        /// <param name="terminals"></param>
        /// <param name="functions"></param>
        /// </summary>>
        public void DnaParseExpressionTestHelper(GenoType testGene, int terminals, int functions) {
            PrimitiveSet primitiveSet = new PrimitiveSet();
            primitiveSet.Parse(testGene);

            //The gene must not change
            Assert.IsNotNull(testGene, "TestProb was null after parse");
            Assert.IsInstanceOfType(testGene, typeof(GenoType), "Type of TestProb has changed");

            Assert.IsNotNull(primitiveSet.ClosureType, "Closure type was null after parse");
            Assert.IsTrue(primitiveSet.ClosureType == typeof(int), "TestProb closure type (i.e. generic class param) should be int, and ClosureType of DnaParse the same");

            Assert.IsNotNull(primitiveSet.TerminalSet, "Terminal set was null after parse");
            Assert.IsNotNull(primitiveSet.FunctionSet, "Function set was null after parse");

            Assert.IsNotNull(primitiveSet.BooleanSets(), "Boolean set was null after parse");
            Assert.IsTrue(primitiveSet.BooleanSets().FunctionSet.Count == 0, "Boolean function set found. Only ints expected");
            Assert.IsTrue(primitiveSet.BooleanSets().TerminalSet.Count == 0, "Boolean terminal set found. Only ints expected");

            Assert.IsNotNull(primitiveSet.CommandSets(), "Commands set was null after parse");
            Assert.IsTrue(primitiveSet.CommandSets().FunctionSet.Count == 0, "Command function set found. Only ints expected");
            Assert.IsTrue(primitiveSet.CommandSets().TerminalSet.Count == 0, "Command terminal set found. Only ints expected");

            Assert.IsTrue(primitiveSet.TerminalSet.Count == terminals, "Terminal set count was " + primitiveSet.TerminalSet.Count + "Check that TestProb class does indeed have " + terminals + " terminals");
            Assert.IsTrue(primitiveSet.FunctionSet.Count == functions, "Function set count was " + primitiveSet.FunctionSet.Count + "Check that TestProb class does indeed have " + functions + " terminals");

            Assert.IsNotNull(primitiveSet.TypedSet(), "Typed sets was null");
            Assert.IsTrue(primitiveSet.TypedSet().FunctionSet.Count == functions, "Typed function set should have " + functions + " functions");
            Assert.IsTrue(primitiveSet.TypedSet().TerminalSet.Count == terminals, "Typed Terminal set should have " + terminals + " terminals");

            //Check for dublicates
            for (int i = 0; i < primitiveSet.TerminalSet.Count; i++) {
                for (int j = 0; j < primitiveSet.TerminalSet.Count; j++) {
                    Assert.IsFalse(primitiveSet.TerminalSet[i] == primitiveSet.TerminalSet[j] && i != j);
                }
            }

            //Check for dublicates
            for (int i = 0; i < primitiveSet.FunctionSet.Count; i++) {
                for (int j = 0; j < primitiveSet.FunctionSet.Count; j++) {
                    Assert.IsFalse(primitiveSet.FunctionSet[i] == primitiveSet.FunctionSet[j] && i != j);
                }
            }

            //Print the result
            foreach (Terminal terminal in primitiveSet.TerminalSet) {
                Console.WriteLine(terminal);
            }
            foreach (Function function in primitiveSet.FunctionSet) {
                Console.WriteLine(function);
            }

        }

        /// <summary>
        /// Features of DnaParse must not be null after a parse.
        /// 
        /// Closure type must always be bool
        /// 
        /// Command functions must always exist i.e. IF_THEN_ELSE, SEQUENCE and any n-arity boolean methods.
        /// 
        /// Terminals set consists of terminal bools and terminal voids. 
        /// 
        /// Terminals bools and functions should be the same when give a boolean set.
        /// 
        /// After a parse there should be no dublicates in the sets.
        /// 
        /// A 0 argument for bool terminal set (ie zero bool PrimitiveSet fields or zero arity functions) should result in a terminal set with two elements - a bool true and false should be added.
        /// </summary>
        /// <param name="testGene"></param>
        /// <param name="boolterminals"></param>
        /// <param name="boolfunctions"></param>
        /// <param name="voidTerminals"></param>
        public void DnaParseCommandTestHelper(GenoType testGene, int boolterminals, int boolfunctions, int voidTerminals) {
            PrimitiveSet cp = new PrimitiveSet();
            cp.Parse(testGene);

            //Equalivant to no bool terminals faound in GenoType class. We want two as the result in this case, a true and a false boolean constant.
            if (boolterminals == 0)
                boolterminals = 2;

            //The gene must not change
            Assert.IsNotNull(testGene, "TestProb was null after parse");
            Assert.IsInstanceOfType(testGene, typeof(GenoType), "Type of TestProb has changed");

            Assert.IsNotNull(cp.ClosureType, "Closure type was null after parse");
            Assert.IsTrue(cp.ClosureType == typeof(bool), "Closure type for command genotype should be bool");

            Assert.IsNotNull(cp.TerminalSet, "Terminal set was null after parse");
            Assert.IsNotNull(cp.FunctionSet, "Function set was null after parse");

            Assert.IsNotNull(cp.BooleanSets(), "Boolean set was null after parse");
            Assert.IsTrue(cp.BooleanSets().TerminalSet.Count == boolterminals, "Boolean set is missing terminals");

            Assert.IsNotNull(cp.CommandSets(), "Commands set was null after parse");
            Assert.IsTrue(0 < cp.CommandSets().FunctionSet.Count, "No command functions found");
            Assert.IsTrue(cp.CommandSets().TerminalSet.Count == voidTerminals, "Missing void terminals");

            Assert.IsTrue(cp.TerminalSet.Count == boolterminals + voidTerminals, "Terminal set count was " + cp.TerminalSet.Count + "Check that TestProb class does indeed have " + boolterminals + voidTerminals + " terminals");
            Assert.IsTrue(0 < cp.FunctionSet.Count, "Function set count was " + cp.FunctionSet.Count + " the must be functions, e.g. IF_THEN_ELSE ");

            Assert.IsNotNull(cp.TypedSet(), "Typed sets was null");
            Assert.IsTrue(cp.TypedSet().FunctionSet.Count == boolfunctions, "Typed function set should have " + boolfunctions + "functions");
            Assert.IsTrue(cp.BooleanSets().TerminalSet.Count == boolterminals, "Typed Terminal set should have " + boolterminals + " terminals");

            //Check for dublicates
            for (int i = 0; i < cp.TerminalSet.Count; i++) {
                for (int j = 0; j < cp.TerminalSet.Count; j++) {
                    Assert.IsFalse(cp.TerminalSet[i] == cp.TerminalSet[j] && i != j);
                }
            }

            //Check for dublicates
            for (int i = 0; i < cp.FunctionSet.Count; i++) {
                for (int j = 0; j < cp.FunctionSet.Count; j++) {
                    Assert.IsFalse(cp.FunctionSet[i] == cp.FunctionSet[j] && i != j);
                }
            }

            //Print the result
            foreach (Terminal terminal in cp.TerminalSet) {
                Console.WriteLine(terminal);
            }
            foreach (Function function in cp.FunctionSet) {
                Console.WriteLine(function);
            }

        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentNullException), "Dnaparser did not catch null argument.")]
        public void NullUserIdInConstructor() {
            new PrimitiveSet().Parse(null);
        }
    }
}
