﻿using System;
using System.Collections.Generic;
using Flatland.GeneticProgramming.CoreTypes;
using Flatland.GeneticProgramming.Initializations;
using Flatland.GeneticProgramming.Selections;
using Flatland.GeneticProgramming.SyntaxTree;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test {
    [TestClass]
    public class SelectionTest : TestHelper {
        /// <summary>
        /// Given a list of reports selection must return any given number of reports subject to some fitness evaluation.
        /// It is no requirement how fitness is meassured or how many individuals are selected.
        /// After selection reports, genes and trees returned must be not null and unique.
        /// PrimitiveSet must not be null.
        /// Selection shold not modify the tree
        /// Fitness value for alle test population goes from 0 to 9
        /// </summary>
        [TestMethod]
        public void SelectionsTest() {
            Initialization init = new Initialization(InitializationTechnique.Full);
            foreach (SelectionTechnique selectionTechnique in Enum.GetValues(typeof(SelectionTechnique))) {
                Selection selection = new Selection(selectionTechnique);
                //Arbitrary sized population
                int popSize = 10;
                List<Report> reps = new List<Report>(popSize);
                Console.WriteLine("Fitness population before selection: ");
                for (int i = 0; i < popSize; i++) {
                    GenoType gene = new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test();
                    PrimitiveSet primitiveSet = new PrimitiveSet();
                    primitiveSet.Parse(gene);
                    Tree t1 = init.Init((uint)3, primitiveSet);
                    gene.Tree = t1;
                    reps.Add(new Report { GenoType = gene, PrimitiveSet = primitiveSet, Fitness = i });
                    Console.Write(i + ",");
                }
                List<Report> newPop = selection.Select(reps);
                Assert.IsNotNull(newPop, "New population was null");
                Console.WriteLine("\nFitness population after selection. Method: " + selectionTechnique);
                foreach (var report in newPop) {
                    Console.Write(report.Fitness + ",");
                    Assert.IsNotNull(report.PrimitiveSet, "PrimitiveSet was null after selection");
                    Assert.IsNotNull(report.GenoType, "GenoType was null after selection");
                    Assert.IsNotNull(report.GenoType.Tree, "Tree was null after selection");
                    CheckAllHasParent(report.GenoType.Tree);
                    CheckCorrectPrimitivesInNodes(report.GenoType.Tree);
                }
                CheckPopulationForUniqueNodes(newPop);
            }
        }
    }
}
