﻿using System;
using System.Reflection.Emit;
using Flatland.GeneticProgramming.CoreTypes;
using Flatland.GeneticProgramming.Initializations;
using Flatland.GeneticProgramming.SyntaxTree;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test {
    [TestClass]
    public class TreeTest : TestHelper{

        public bool b;
        public bool[] bools = new bool[1];

        ////////////////////////////////////////////////////////////////////////////////////////////////////
        /// <summary>   Equalities this object. </summary>
        /// Equality checks that equality holds for the different combinations of the command GenoType that
        /// will occur in the tree.
        /// <remarks>   Anders, 29-07-2012. </remarks>
        ////////////////////////////////////////////////////////////////////////////////////////////////////
        [TestMethod]
        public void EqualityTest()
        {
            Node ite = new Node(new Function(SetType.IF_THEN_ELSE));            
            Node seq = new Node(new Function(SetType.SEQUENCE));

            Node param_bool_func1 = new Node(new Function(new DynamicMethod("BoolFunc1", typeof(bool), new Type[] { typeof(double) })));
            Node param_bool_func2 = new Node(new Function(new DynamicMethod("BoolFunc2", typeof(bool), new Type[] { typeof(double), typeof(double) })));

            Leaf zero_void_func = new Leaf(new Terminal(new DynamicMethod("VoidMethod", typeof (void), new Type[] {})));
            Leaf zero_bool_func = new Leaf(new Terminal(new DynamicMethod("BoolMethod", typeof(bool), new Type[] { })));

            Leaf bool_var = new Leaf(new Terminal(GetType().GetField("b"), "b"));
            Leaf true_const = new Leaf(new Terminal(true));
            Leaf false_const = new Leaf(new Terminal(false));
            Leaf bool_param = new Leaf(new Terminal(GetType().GetField("bools"), 0));

            /*
             *  IF_THEN_ELSE
             */
            Assert.IsTrue(ite.Equals(ite));
            Assert.IsTrue(ite.Equals(seq));
            Assert.IsTrue(ite.Equals(zero_void_func));
            Assert.IsFalse(ite.Equals(param_bool_func1));
            Assert.IsFalse(ite.Equals(param_bool_func2));
            Assert.IsFalse(ite.Equals(zero_bool_func));
            Assert.IsFalse(ite.Equals(true_const));
            Assert.IsFalse(ite.Equals(zero_bool_func));
            Assert.IsFalse(ite.Equals(false_const));
            Assert.IsFalse(ite.Equals(bool_param));

            /*
             * SEQUENCE
             */
            Assert.IsTrue(seq.Equals(ite));
            Assert.IsTrue(seq.Equals(seq));
            Assert.IsTrue(seq.Equals(zero_void_func));
            Assert.IsFalse(seq.Equals(param_bool_func1));
            Assert.IsFalse(seq.Equals(param_bool_func2));
            Assert.IsFalse(seq.Equals(zero_bool_func));
            Assert.IsFalse(seq.Equals(true_const));
            Assert.IsFalse(seq.Equals(zero_bool_func));
            Assert.IsFalse(seq.Equals(false_const));
            Assert.IsFalse(seq.Equals(bool_param));

            /*
             *  Bool param
             */
            Assert.IsFalse(param_bool_func1.Equals(ite));
            Assert.IsFalse(param_bool_func1.Equals(seq));
            Assert.IsFalse(param_bool_func1.Equals(zero_void_func));
            Assert.IsTrue(param_bool_func1.Equals(param_bool_func1));
            Assert.IsTrue(param_bool_func1.Equals(param_bool_func2));
            Assert.IsTrue(param_bool_func1.Equals(zero_bool_func));
            Assert.IsTrue(param_bool_func1.Equals(true_const));
            Assert.IsTrue(param_bool_func1.Equals(zero_bool_func));
            Assert.IsTrue(param_bool_func1.Equals(false_const));
            Assert.IsTrue(param_bool_func1.Equals(bool_param));

            /*
             * Bool param 2
             */
            Assert.IsFalse(param_bool_func2.Equals(ite));
            Assert.IsFalse(param_bool_func2.Equals(seq));
            Assert.IsFalse(param_bool_func2.Equals(zero_void_func));
            Assert.IsTrue(param_bool_func2.Equals(param_bool_func1));
            Assert.IsTrue(param_bool_func2.Equals(param_bool_func2));
            Assert.IsTrue(param_bool_func2.Equals(zero_bool_func));
            Assert.IsTrue(param_bool_func2.Equals(true_const));
            Assert.IsTrue(param_bool_func2.Equals(zero_bool_func));
            Assert.IsTrue(param_bool_func2.Equals(false_const));
            Assert.IsTrue(param_bool_func2.Equals(bool_param));

            /*
             *  Zero void func
             */
            Assert.IsTrue(zero_void_func.Equals(ite));
            Assert.IsTrue(zero_void_func.Equals(seq));
            Assert.IsTrue(zero_void_func.Equals(zero_void_func));
            Assert.IsFalse(zero_void_func.Equals(param_bool_func1));
            Assert.IsFalse(zero_void_func.Equals(param_bool_func2));
            Assert.IsFalse(zero_void_func.Equals(zero_bool_func));
            Assert.IsFalse(zero_void_func.Equals(true_const));
            Assert.IsFalse(zero_void_func.Equals(zero_bool_func));
            Assert.IsFalse(zero_void_func.Equals(false_const));
            Assert.IsFalse(zero_void_func.Equals(bool_param));

            /*
             *  Zero bool func
             */
            Assert.IsFalse(zero_bool_func.Equals(ite));
            Assert.IsFalse(zero_bool_func.Equals(seq));
            Assert.IsFalse(zero_bool_func.Equals(zero_void_func));
            Assert.IsTrue(zero_bool_func.Equals(param_bool_func1));
            Assert.IsTrue(zero_bool_func.Equals(param_bool_func2));
            Assert.IsTrue(zero_bool_func.Equals(zero_bool_func));
            Assert.IsTrue(zero_bool_func.Equals(true_const));
            Assert.IsTrue(zero_bool_func.Equals(zero_bool_func));
            Assert.IsTrue(zero_bool_func.Equals(false_const));
            Assert.IsTrue(zero_bool_func.Equals(bool_param));

            /*
             *  false const
             */
            Assert.IsFalse(false_const.Equals(ite));
            Assert.IsFalse(false_const.Equals(seq));
            Assert.IsFalse(false_const.Equals(zero_void_func));
            Assert.IsTrue(false_const.Equals(param_bool_func1));
            Assert.IsTrue(false_const.Equals(param_bool_func2));
            Assert.IsTrue(false_const.Equals(zero_bool_func));
            Assert.IsTrue(false_const.Equals(true_const));
            Assert.IsTrue(false_const.Equals(zero_bool_func));
            Assert.IsTrue(false_const.Equals(false_const));
            Assert.IsTrue(false_const.Equals(bool_param));


            /*
             *  true const
             */
            Assert.IsFalse(true_const.Equals(ite));
            Assert.IsFalse(true_const.Equals(seq));
            Assert.IsFalse(true_const.Equals(zero_void_func));
            Assert.IsTrue(true_const.Equals(param_bool_func1));
            Assert.IsTrue(true_const.Equals(param_bool_func2));
            Assert.IsTrue(true_const.Equals(zero_bool_func));
            Assert.IsTrue(true_const.Equals(true_const));
            Assert.IsTrue(true_const.Equals(zero_bool_func));
            Assert.IsTrue(true_const.Equals(false_const));
            Assert.IsTrue(true_const.Equals(bool_param));

            /*
            *  true const
            */
            Assert.IsFalse(bool_param.Equals(ite));
            Assert.IsFalse(bool_param.Equals(seq));
            Assert.IsFalse(bool_param.Equals(zero_void_func));
            Assert.IsTrue(bool_param.Equals(param_bool_func1));
            Assert.IsTrue(bool_param.Equals(param_bool_func2));
            Assert.IsTrue(bool_param.Equals(zero_bool_func));
            Assert.IsTrue(bool_param.Equals(true_const));
            Assert.IsTrue(bool_param.Equals(zero_bool_func));
            Assert.IsTrue(bool_param.Equals(false_const));
            Assert.IsTrue(bool_param.Equals(bool_param));

        }

        /// <summary>
        /// A tree is logically equal to its clone, but must not share any reference.
        /// Tree must be valid. 
        /// </summary>
        [TestMethod]
        public void CloneTreeTestHelper() {
            Console.WriteLine("Checking clone tree");
            PrimitiveSet testClass = new PrimitiveSet();
            testClass.Parse(new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test());
            int maxTestDepth = 5;
            foreach (InitializationTechnique initializationTechnique in Enum.GetValues(typeof(InitializationTechnique)))
            {
                Initialization init = new Initialization(initializationTechnique);
                for (int i = 0; i < maxTestDepth; i++) {
                    Tree tree = init.Init(5, testClass);
                    Console.WriteLine("Checking parents for TREE with depth {0}", i);
                    CheckAllHasParent(tree);
                    Tree clone = (Tree)tree.Clone();
                    Console.WriteLine("Checking parents for CLONE with depth {0}", i);
                    CheckAllHasParent(clone);
                    Assert.IsTrue(CheckLogicalEqualityInNodes(clone, tree), "Tree and clone tree did not have logical equalivant nodes");
                    Assert.IsTrue(CheckLogicalEqualityInNodes(clone, clone), "Clone tree was not logically equalivant to itself");
                    Assert.IsTrue(CheckLogicalEqualityInNodes(tree, tree), "Tree was not logically equalivant to itself");
                    Assert.IsFalse(clone == tree, "Tree and clone tree did have equal object references");
                    Assert.IsFalse(CheckHasSharedReference(clone, tree), "Tree and clone tree did have equal object references");
                    //The above should assert equal depth but we recheck this just for tricks
                    Assert.IsTrue(tree.MaxDetpth() == clone.MaxDetpth(), "Tree and clone had different depth");
                }
            }
        }

        [TestMethod]
        public void DepthTest1()
        {
            Initialization init = new Initialization(InitializationTechnique.Full);
            PrimitiveSet primitiveSet = new PrimitiveSet();
            primitiveSet.Parse(new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test());
            Tree t = init.Init(3, primitiveSet);
            Assert.IsTrue(t.MaxDetpth() == 3);
            Assert.IsTrue(t.Depth() == 1);
            Assert.IsTrue(((Node)t).Children[0].MaxDetpth() == 2);
            Assert.IsTrue(((Node)t).Children[0].Depth() == 2);
            Assert.IsTrue(((Node)((Node)t).Children[0]).Children[0].MaxDetpth() == 1);
            Assert.IsTrue(((Node)((Node)t).Children[0]).Children[0].Depth() == 3);
        }

        [TestMethod]
        public void DepthTest2() {
            Initialization init = new Initialization(InitializationTechnique.Full);
            PrimitiveSet primitiveSet = new PrimitiveSet();
            primitiveSet.Parse(new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test());
            Tree t = init.Init(3, primitiveSet);
            Assert.IsTrue(t.MaxDetpth() == 3);
            Assert.IsTrue(t.Depth() == 1);
            Assert.IsTrue(((Node)t).Children[0].MaxDetpth() == 2);
            Assert.IsTrue(((Node)t).Children[0].Depth() == 2);
            Assert.IsTrue(((Node)((Node)t).Children[0]).Children[1].MaxDetpth() == 1);
            Assert.IsTrue(((Node)((Node)t).Children[1]).Children[1].Depth() == 3);
        }

        [TestMethod]
        public void DepthTest3() {
            Initialization init = new Initialization(InitializationTechnique.Full);
            PrimitiveSet primitiveSet = new PrimitiveSet();
            primitiveSet.Parse(new Flatland.GeneticProgramming.TestProblems_Do_not_modify.Test());
            Tree t = init.Init(1, primitiveSet);
            Assert.IsTrue(t.MaxDetpth() == 1);
            Assert.IsTrue(t.Depth() == 1);
        }
    }
}
